#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "sndconf.h"
#include "oscillators.h"



/*** Main ***/
int main(int argc, char* argv[])
{
  /* Pointer to wave-generating function */
  float (*fwave)(float, float, float) = &sine;

  /* Pointer to oscillator function */
  void (*fosc)(float(*fwave)(float, float, float), float, float, float) = &osc_mono;

  /* Sound configuration structure, containing default values */
  sndparams.channels = 1;
  sndparams.rate = 8000;
  sndparams.write_wave = 0;

  /* Oscillator volume */
  float vol = 0.4f;

  /* File pointer */
  FILE* fp = stdin;

  /* Display help if requested */
  if (argc == 2 && (strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-h") == 0)) {
    printf("%s: cynth oscillator\n", argv[0]);
    printf("Usage: %s [OPTION]... [FILE]\nOptions:\n", argv[0]);
    printf("-h, --help\t\thelp\n");
    printf("-w, --waveform TYPE\twaveform (sine (default), square, triangle, saw)\n");
    printf("-h16, --harmonics16\tadd 16 upper harmonics\n");
    printf("-V, --volume\t\toscillator volume [0.0;1.0] (default 0.4)\n");
    printf("-c, --channels\tnumber of channels (default 1)\n");
    printf("-f, --format\t\tsample format (U8 (default) or U16_LE)\n");
    printf("-r, --rate\t\tsample rate [Hz] (default 8000)\n");
    printf("-W, --writewate\tsave waveform to wave.dat\n");
    printf("If FILE is not specified, input will be read from stdin.\n");
    return 0; /* Exit with success */
  }

  /* Process input parameters */
  int i;
  for (i = 1; i < argc; ++i) {	/* Skip argv[0] */

    /* Waveform specified */
    if (strcmp(argv[i], "-w") == 0 || strcmp(argv[i], "--waveform") == 0) {
      if (strcmp(argv[i+1], "square") == 0) {
	fwave = &square; ++i;
      } else if (strcmp(argv[i+1], "triangle") == 0) {
	fwave = &triangle; ++i;
      } else if (strcmp(argv[i+1], "saw") == 0) {
	fwave = &saw; ++i;
      }

      /* Use different oscillator function */
    } else if (strcmp(argv[i], "-h16") == 0 || strcmp(argv[i], "--harmonics16") == 0) {
      fosc = &osc_mono_16h;

      /* Set oscillator volume */
    } else if (strcmp(argv[i], "-V") == 0 || strcmp(argv[i], "--volume") == 0) {
      vol = atof(argv[i+1]); ++i;

      /* Set number of channels */
    } else if (strcmp(argv[i], "-c") == 0 || strcmp(argv[i], "--channels") == 0) {
      sndparams.channels = atoi(argv[i+1]); ++i;

      /* Set sample format */
    } else if (strcmp(argv[i], "-f") == 0 || strcmp(argv[i], "--format") == 0) {
      if (strcmp(argv[i+1], "U16_LE") == 0)
	;/*typedef unsigned short FORMAT_TYPE; ++i;*/

      /* Set sample rate */
    } else if (strcmp(argv[i], "-r") == 0 || strcmp(argv[i], "--rate") == 0) {
      sndparams.rate = atoi(argv[i+1]); ++i;

      /* Write waveform to wave.dat */
    } else if (strcmp(argv[i], "-W") == 0 || strcmp(argv[i], "--writewave") == 0) {
      sndparams.write_wave = 1;

      /* File given as input argument */
    } else if (i == argc-1) {
      if ((fp = fopen(argv[i], "r")) == NULL) {
	fprintf(stderr, "%s: Can't open %s for read\n", argv[0], argv[i]);
	exit(1); /* Exit with error */
      }
    } 
  }

  /* Read input data, line by line and call oscillator function */
  float freq, time;
  while (!feof(fp) && fscanf(fp, "%f\t%f", &freq, &time) == 2) { 
    fosc(fwave, freq, time, vol);
  }

  /* Flush buffers and return successfully */
  exit(0);
}
